// =============================================================================
// File: synchi.js
// Version: 3.0
// 
// Initializes synchi and codemirror code editor.
// =============================================================================

$ = jQuery;
var synchi_editor = false;
var synchi_editor_line = null;
var synchi_default_controls = [
    'find_prev','search','find_next','search_replace','spacer',
    'indent_left','format','indent_right','spacer',
    'undo','redo','spacer',
    'goto','fullscreen'
];
var synchi_fullscreen = false;
var synchi_saving = false;
var synchi_fileInEdit = 'unknown';

// enable concat function for JSON objects
if (!$.concat) $.extend({
    concat: function(a, b) { 
        var merged = {};
        for (var attr_a in a) merged[attr_a] = a[attr_a];
        for (var attr_b in b) merged[attr_b] = b[attr_b];
        return merged;
    }
});

// enable merge function for arrays
if (!$.merge) $.extend({
    marge: function(a, b) { 
        var merged = [];
        for (var attr_a in a) merged[merged.length] = a[attr_a];
        for (var attr_b in b) merged[merged.length] = b[attr_b];
        return merged;
    }
});

/**
 * Returns loading image HTML
 * 
 * @return image string
 */
function synchi_getLoadingImageHTML() {
    var src = synchi_path + 'loading.gif';
    return '<img src="'+src+'" border="0" />';
}

/**
 * Renders Synchi Controls
 * 
 * @param controls array of controls or false
 */
function synchi_RenderControls(controls) {
    if(!controls) return;
    var div = $('<div id="synchi_controls"></div>');
    var i = 0;
    for(i=0; i<controls.length; i++) {
        var control = controls[i];
        var src = synchi_path + 'icons/' + control + '.png';
        if(control == 'spacer') {
             div.append(
                '<a href="#" onclick="return false;" class="synchi_spacer">' + 
                    '<img src="'+src+'" border="0" />'+
                '</a>'
            );
            continue;
        }
        var title = '';
        switch(control) {
            case 'save' :title = 'Save (Alt+Shit+S)';break;
            case 'search' :title = 'Search (Ctrl+F)';break;
            case 'find_prev' :title = 'Find Previous (Ctrl+G)';break;
            case 'find_next' :title = 'Find Next (Shift+Ctrl+G)';break;
            case 'search_replace' :title = 'Replace (Shift-Ctrl-R)';break;
            case 'format' :title = 'Format (Alt+Shift+F)';break;
            case 'indent_left' :title = 'Indent left (Ctrl+[)';break;
            case 'indent_right' :title = 'Indent right (Ctrl+])';break;
            case 'undo' :title = 'Undo (Ctrl+Z)';break;
            case 'redo' :title = 'Redo (Ctrl+Y)';break;
            case 'goto' :title = 'Jump to line (Alt+Shift+L)';break;
            case 'fullscreen' :title = 'Toggle Fullscreen (Alt+Shift+Enter)';break;
            //case 'comment' :title = 'Comment Selection (Alt+Shift+/)';break;
            default :title = 'action';
        }
        div.append(
            '<a href="#" onclick="synchi_Control(\''+control+'\'); return false;" title="'+title+'">' + 
                '<img src="'+src+'" border="0" />'+
            '</a>'
        );
    }
    $('.CodeMirror-scroll').after(div);
}

/**
 * Creates synchi (codemirror) editor
 * 
 * @param element jQuery wrappped textarea
 * @param options JSON options
 * @param controls array of controls to appear (if false no controls will be used)
 */
function synchi_CreateEditor(element,options,controls) {
    options = $.concat(synchi_settings,options);
    synchi_editor = CodeMirror.fromTextArea(element.get(0),$.concat(options,{
        tabMode: "indent",
        onCursorActivity: function() {
            synchi_editor.setLineClass(synchi_editor_line, null, null);
            synchi_editor_line = synchi_editor.setLineClass(
                synchi_editor.getCursor().line, 
                null, "synchi_activeline"
            );
        },
        onChange: function(){synchi_editor.save();}
    }));
    
    // set first line active line
    synchi_editor_line = synchi_editor.setLineClass(0, "synchi_activeline");
    if(!controls) controls = false;
    synchi_RenderControls(controls);
    synchi_editor.focus();
    
    // attach keyup events
    $('.CodeMirror').keyup(function(event){
        var keycode = 0;        
        if(event.keyCode) keycode = event.keyCode;
        else if(event.which) keycode = event.which;
        
        switch(keycode) {
            case 13: // ENTER
                if(!event.altKey) return;
                if(!event.shiftKey) return;
                synchi_ToggleFullscreen();
                break;
            case 27: // ESC
                if(synchi_fullscreen) synchi_ToggleFullscreen();
                break;
            case 70: // f
                if(!event.altKey) return;
                if(!event.shiftKey) return;
                synchi_Control('format');
                break;
            case 76: // l
                if(!event.altKey) return;
                if(!event.shiftKey) return;
                synchi_GoToLine();
                break;
            case 83:
                if(!event.altKey) return;
                if(!event.shiftKey) return;
                synchi_save();
                break;
//            case 191: // /
//                if(!event.altKey) return;
//                if(!event.shiftKey) return;
//                synchi_Control('comment');
        }
        //event.preventDefault();
    });    
}

/**
 *Performs a synchi control action
 *
 * @param control string control name
 */
function synchi_Control(control) {
    switch(control) {
        case 'save':
            synchi_save();
            break;
        case 'search':
            CodeMirror.commands.find(synchi_editor);
            break;
        case 'find_prev':
            CodeMirror.commands.findPrev(synchi_editor);
            break;
        case 'find_next':
            CodeMirror.commands.findNext(synchi_editor);
            break;
        case 'search_replace':
            CodeMirror.commands.replaceAll(synchi_editor);
            break;
        case 'undo':
            synchi_editor.undo();
            break;
        case 'redo':
            synchi_editor.redo();
            break;
        case 'goto':
            synchi_GoToLine();
            break;
        case 'format':
            synchi_editor.indentSelection('smart');
            break;
        case 'indent_left':
            synchi_editor.indentSelection('subtract');
            break;
        case 'indent_right':
            synchi_editor.indentSelection('add'); 
            break;
//        case 'comment':
//            synchi_editor.autoFormatRange(synchi_editor.getCursor(true), synchi_editor.getCursor(false));
//            break;
        case 'fullscreen':
            synchi_ToggleFullscreen();
            break;
    }
}

/**
 * Scrolls synchi editor to a certain line
 */
function synchi_GoToLine() {
    synchi_editor.openDialog('Jump to line: <input type="text" style="width: 10em"/>',
    function(line){
        if(!line) return;
        line = Number(line-1);
        if(!synchi_editor.getLineHandle(line)) return;
        synchi_editor.setCursor(line,0);
        synchi_editor.setSelection(
            {line:line,ch:0},
            {line:line+1,ch:0}
        );
        synchi_editor.focus();
    });    
}

/**
 * Toggles fullscreen editing mode
 */
function synchi_ToggleFullscreen() {
    var cm = $('.CodeMirror');
    if(cm.hasClass('synchi_fullscreen')) {
        cm.removeClass('synchi_fullscreen');
        synchi_fullscreen = false;
    }
    else {
        cm.addClass('synchi_fullscreen');
        synchi_fullscreen = true;
        synchi_showMessage('Full Screen Editing is On. ESC to close.',1500);
        synchi_editor.focus();
    }
}

/**
 * Checks if TinyMCE is being used
 * 
 * @return true if TinyMCE is being used
 */
function synchi_isTinyMCE() {
    var tinymce = $("#content-tmce");
    return tinymce.length != 0;
}

/**
 * Initializes themes/plugins editor with syntax highlight
 * 
 * @param fileedit jQuery wrapped textarea object
 */
function synchi_initEditor(fileedit) {
    var extension = false;
    var editor = $('#newcontent');
    if(fileedit.find('h3').length != 0) {
        var h3 = fileedit.find('h3');
        if(h3.find('span').length != 0) extension = h3.find('span').text().replace('(','').replace(')','');
        else extension = h3.text();
    }
    else if(fileedit.find('big strong').length != 0)extension = fileedit.find('big strong').text();
    else return;
    
    // save file in edit
    synchi_fileInEdit = extension;
    
    // clean extension
    extension = $(extension.split('.')).last().get(0);
    
    // init mode
    var controls = $.merge(['save', 'spacer'],synchi_default_controls);

    // determine mode
    switch(extension) {
        case 'css':
            synchi_CreateEditor(editor,{mode:'css'},controls);
            break;
        case 'js':
            synchi_CreateEditor(editor,{mode:'javascript'},controls);
            break;
        case 'html': case 'htm':
            synchi_CreateEditor(editor,{mode:'text/html'},controls);
            break;
        case 'sql':
            synchi_CreateEditor(editor,{mode:'mysql'},controls);
            break;
        case 'php':
            synchi_CreateEditor(editor,{
                matchBrackets: true,
                mode: "application/x-httpd-php",
                indentUnit: 4,
                indentWithTabs: false,
                enterMode: "keep"
            },controls);
            break;
        case 'xml':
            synchi_CreateEditor(editor,{mode:'xml'},controls);
            break;
        default:return;
    }
}

/**
 * Initializes article editor with syntax highlight
 * 
 * @param editor_area jQuery wrapped textarea object
 */
function synchi_initArticleEditor(editor_area) {
    // swich to HTML editor if TinyMCE is being used (thank you Peter Mukhortov)
    if(synchi_isTinyMCE()) switchEditors.switchto($("#content-html").get(0));
        
    // init the editor
    synchi_CreateEditor(editor_area,{
        mode:'text/html'
    },synchi_default_controls);    
        
    // handle TinyMCE
    if(synchi_isTinyMCE()) {
        $("#content-tmce").click(function(){
            if($("#wp-content-wrap").hasClass('tmce-active')) return;
            // revert to textarea
            if(synchi_editor) {
                synchi_editor.toTextArea();
                synchi_editor = false;
            }
            // default behaviour
            switchEditors.switchto($(this).get(0));
            return false;
        });
        $("#content-html").click(function(){
            if($("#wp-content-wrap").hasClass('html-active')) return;
            // default behaviour
            switchEditors.switchto($(this).get(0));
            // re-init article editor
            synchi_CreateEditor($("#content"),{
                mode:'text/html'
            },synchi_default_controls);    
            return false;
        });
    }
        
    // handle fullscreen
    $("#content_wp_fullscreen").live('click',function(){
        // default behaviour
        fullscreen.on();
        // added behaviour
        fullscreen.switchmode('tinymce');
        return false;
    });
    $("#qt_content_fullscreen").live('click',function(){
        // revert to textarea
        if(synchi_editor) {
            synchi_editor.toTextArea();
            synchi_editor = false;
        }
        // default behaviour
        fullscreen.on();
        // added behaviour
        fullscreen.switchmode('html');
        // init fullscreen editor
        synchi_CreateEditor($("#wp_mce_fullscreen"),{
            mode:'text/html'
        },synchi_default_controls);
        return false;
    });
    $("#wp-fullscreen-close").live('click',function(){
        // revert to textarea
        if(synchi_editor) {
            synchi_editor.toTextArea();
            synchi_editor = false;
        }
        // default behaviour
        fullscreen.off();
        // init editor
        synchi_CreateEditor($("#content"),{
            mode:'text/html'
        },synchi_default_controls);
        return false;
    });
    $('#wp-fullscreen-modes a').live('click',function(){
        var index = $('#wp-fullscreen-modes a').index($(this).get(0));
        switch(index) {
            case 0:
                if(synchi_editor) {
                    synchi_editor.toTextArea();
                    synchi_editor = false;
                }
                // default behaviour
                fullscreen.switchmode('tinymce');
                $('#wp-fullscreen-container textarea').hide();
                return false;
            case 1:
                // default behaviour
                fullscreen.switchmode('html');
                // init fullscreen editor
                synchi_CreateEditor($("#wp_mce_fullscreen"),{
                    mode:'text/html'
                },synchi_default_controls);
                return false;
        }
    });
}

/**
 * Shows a floating transparent message
 * 
 * @param text message text
 * @param timeout time to wait and hide (if null, message stays)
 */
function synchi_showMessage(text,timeout) {
    var message = $('<div class="synchi_mesageBox"><span>'+text+'</span></div>');
    $('body').append(message);
    if(timeout) setTimeout(synchi_hideMessage,timeout);
}

/**
 * Hides the floating transparent message
 * 
 * @param callback perform after hiding
 */
function synchi_hideMessage(callback) {
    var message = $('.synchi_mesageBox');
    if(message.length == 0) return;
    message.fadeOut(300, function(){ 
        message.remove();
        if(callback) callback();
    });
}

/**
 * Saves the document in edit
 */
function synchi_save() {
    if(synchi_saving) return;
    synchi_saving = true;
    synchi_showMessage(synchi_getLoadingImageHTML()+'<br />Saving...');
    var form = $("#template");
    if(form.length == 0) {
        synchi_saving = false;
        return;
    }
    $.ajax({
        type:'POST',
        url: form.attr('action'),
        data:form.serialize(), 
        success: function() {
            synchi_hideMessage(function(){
                synchi_showMessage('"' + synchi_fileInEdit + '" saved.',1000);
                synchi_saving = false;
            });
        },
        error: function() {
            synchi_hideMessage(function(){
                synchi_showMessage('Error: "' + synchi_fileInEdit + '" not saved.',2000);
                synchi_saving = false;
            });
        }
    });
}

// On Load
$(function(){
    var fileedit = $(".fileedit-sub div.alignleft");
    var editor_area = $("#content");
    if(fileedit.length != 0) synchi_initEditor(fileedit);
    else if(editor_area.length != 0) synchi_initArticleEditor(editor_area);
});